// $Id: Disjunction.java,v 1.6 2004/02/27 16:30:00 m31 Exp $
//
// This file is part of the program suite "Principia Mathematica II"
// which is a working prototype for the main project:
// "Hilbert II" - http://www.qedeq.org
//
// Copyright 2001-2004  Michael Meyling <mime@qedeq.org>.
//
// "Principia Mathematica II" is free software; you can redistribute
// it and/or modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.

package com.meyling.principia.logic.basic;


import com.meyling.principia.argument.Argument;
import com.meyling.principia.argument.ArgumentException;


/**
 * Disjunction of formulas. Implementation of the locigal &quot;or&quot;.
 *
 * @version $Revision: 1.6 $
 * @author  Michael Meyling
 */
public class Disjunction extends LogicalOperatorWithTwoFormulas {


    /**
     * Constructs a logical disjunction.
     *
     * @param   arguments     formulas to operate on
     * @throws  ArgumentException    if there are not exactly two
     *          arguments that are instances of {@link Formula} or
     *          if the is a subject variable that would exist
     *          bound and unbound in the resulting formula
     *
     */
    public Disjunction(final Argument[] arguments)
            throws ArgumentException {
        super (arguments);
    }


    /**
     * Constructs a logical disjunction.
     *
     * @param   formula1    first formula
     * @param   formula2    second formula
     * @throws  ArgumentException    if the is a subject variable
     *          that would exist bound and unbound in the resulting formula
     *
     */
    public Disjunction(final Formula formula1, final Formula formula2)
            throws ArgumentException {
        super(formula1, formula2);
    }


    /** @inherited */
    public final Argument create(final Argument[] arguments)
            throws ArgumentException {
        return new Disjunction(arguments);
    }


    /** @inherited */
    public final String toString() {
        StringBuffer buffer = new StringBuffer("(");
        for (int i = 0; i < getArgumentSize(); i++) {
            if (i != 0) {
                buffer.append(" v ");
            }
            buffer.append(getArgument(i));
        }
        buffer.append(")");
        return buffer.toString();
    }


}
