// $Id: RegularArgumentList.java,v 1.5 2004/02/27 16:30:00 m31 Exp $
//
// This file is part of the program suite "Principia Mathematica II"
// which is a working prototype for the main project:
// "Hilbert II" - http://www.qedeq.org
//
// Copyright 2001-2004  Michael Meyling <mime@qedeq.org>.
//
// "Principia Mathematica II" is free software; you can redistribute
// it and/or modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.

package com.meyling.principia.logic.basic;


import com.meyling.principia.argument.AbstractArgumentList;
import com.meyling.principia.argument.Argument;
import com.meyling.principia.argument.ArgumentException;


/**
 * A simple collection of arguments.
 *
 * @version $Revision: 1.5 $
 * @author  Michael Meyling
 */
public class RegularArgumentList extends AbstractArgumentList implements Argument {


    /**
     * Constructs an argument list.
     *
     * @param  arguments list of arguments
     * @throws ArgumentException if there are no arguments
     */
    public RegularArgumentList(final Argument[] arguments)
            throws ArgumentException {
        super(arguments);
        if (getArgumentSize() == 0) {
            throw new ArgumentException(
                BasicConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_NUMBER,
                BasicCreator.getName(this.getClass())
                + BasicConstants.COULD_NOT_BE_EMPTY);
        }
    }


    /** @inherited */
    public final Argument create(final Argument[] arguments)
            throws ArgumentException {
        return new RegularArgumentList(arguments);
    }


    /** @inherited */
    public final String toString() {
        StringBuffer buffer = new StringBuffer("(");
        for (int i = 0; i < getArgumentSize(); i++) {
            if (i != 0) {
                buffer.append(", ");
            };
            buffer.append(getArgument(i));
        }
        buffer.append(")");
        return buffer.toString();
    };


}
