// $Id: RuleDeclaration.java,v 1.7 2004/02/27 16:30:00 m31 Exp $
//
// This file is part of the program suite "Principia Mathematica II"
// which is a working prototype for the main project:
// "Hilbert II" - http://www.qedeq.org
//
// Copyright 2001-2004  Michael Meyling <mime@qedeq.org>.
//
// "Principia Mathematica II" is free software; you can redistribute
// it and/or modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.

package com.meyling.principia.logic.paragraph;


import com.meyling.principia.argument.AbstractDynamicArgumentList;
import com.meyling.principia.argument.Argument;
import com.meyling.principia.argument.ArgumentException;
import com.meyling.principia.argument.Text;
import com.meyling.principia.module.Module;
import com.meyling.principia.module.ModuleCreator;


/**
 * Declares a logical rule.
 *
 * @version $Revision: 1.7 $
 * @author  Michael Meyling
 */
public class RuleDeclaration extends AbstractDynamicArgumentList
        implements Argument, ParagraphCheck {


    /**
     * Constructs a mathematical RuleDeclaration.
     *
     * @param   arguments     name of rule, its description and arguments
     * @throws  ArgumentException    if there are not at leas two
     *          arguments or the first argument is not an instance
     *          of {@link Text} or the second argument
     *          is no instance of {@link Text}.
     */
    public RuleDeclaration(final Argument[] arguments)
            throws ArgumentException {
        super(arguments);

        if (arguments.length < 2) {
            throw new ArgumentException(
                ParagraphConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_NUMBER,
                ModuleCreator.getName(this.getClass())
                + ParagraphConstants.OPERATOR_WITH_AT_LEAST_TWO_ARGUMENTS);
        }
        if (!(arguments[0] instanceof Text)) {
            throw new ArgumentException(
                ParagraphConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                ModuleCreator.getName(this.getClass())
                + ParagraphConstants.FIRST_ARGUMENT_TYPE
                + ParagraphConstants.QUOTED_STRING,
                arguments[0]);
        }
        if (!(arguments[1] instanceof Text)) {
            throw new ArgumentException(
                ParagraphConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                ModuleCreator.getName(this.getClass())
                + ParagraphConstants.SECOND_ARGUMENT_TYPE
                + ParagraphConstants.QUOTED_STRING,
                arguments[1]);
        }
    }


    /**
     * Check if declaration is correct.
     *
     * @param   module   module
     * @param   label    label of paragraph
     * @throws  ArgumentException    if proof is not correct
     */
    public final void check(final Module module, final String label)
            throws ArgumentException {
        try {
            module.addRuleDeclaration(label, this, module);
        } catch (IllegalArgumentException e) {
            throw new ArgumentException(
                ParagraphConstants.CODE_METHOD_ARGUMENT_CHECK_FAILED,
                e.getMessage(),
                this.getArgument(0));
        }

    }


    /** @inherited */
    public final Argument create(final Argument[] arguments)
            throws ArgumentException {
        return new RuleDeclaration(arguments);
    }


    /** @inherited */
    public final String toString() {
        StringBuffer buffer = new StringBuffer("RuleDeclaration ");
        for (int i = 0; i < getArgumentSize(); i++) {
            if (i != 0) {
                buffer.append(", ");
            };
            buffer.append(getArgument(i));
        }
        return buffer.toString();
    }


}
